#pragma once

#include "QuantumCircuitElement.h"
#include "QuantumGate.h"
#include "QuantumTerminal.h"
#include "QuantumWire.h"
#include "QuantumState.h"

#include "Vector.h"

class QuantumCircuit
{
public:
	void addInputTerminal(QuantumTerminalPtr terminal);

	void addOutputTerminal(QuantumTerminalPtr terminal);

	void addQuantumWire(QuantumWirePtr wire);

	void addLogicGate(QuantumGatePtr gate);

	void buildWireConnections();

	const Vector<QuantumTerminal*>& getInputTerminals() const;

	const Vector<QuantumTerminal*>& getOutputTerminals() const;

	const Vector<QuantumGate*>& getLogicGates() const;

	const Vector<QuantumWire*>& getQuantumWires() const;

	QuantumState getInputState() const;

private:
	bool connectivityIsValid() const;

	Vector<QuantumTerminal*> mInputTerminals;
	Vector<QuantumTerminal*> mOutputTerminals;

	Vector<QuantumWire*> mWires;
	Vector<QuantumGate*> mGates;

	Vector<Ptr<QuantumCircuitElement> > mElements;
};