#pragma once

#include "QuantumCircuitElement.h"

#include "QuantumWire.h"

#include "Vector.h"

class QuantumGate : public QuantumCircuitElement
{
public:
    enum class GateType
    {
        X,
        Y,
        Z,
        H,
        CNOT,
        CUSTOM
    };
    virtual ~QuantumGate() = default;

    virtual size_t getNumInputs() const = 0;

    virtual size_t getNumOutputs() const = 0;

    virtual AbstractQuantumWire* getInput(size_t idx) const = 0;

    virtual AbstractQuantumWire* getOutput(size_t idx) const = 0;

    virtual void setAtInput(size_t idx, AbstractQuantumWire* value) = 0;

    virtual void setAtOutput(size_t idx, AbstractQuantumWire* value) = 0;

    virtual GateType getGateType() const = 0;

    Type getType() const override
    {
        return Type::GATE;
    }
};
using QuantumGatePtr = Ptr<QuantumGate>;


class NInMOutQuantumGate : public QuantumGate
{
public:
    NInMOutQuantumGate(size_t numIn, size_t numOut, Vector<AbstractQuantumWire*> inputs = {}, Vector<AbstractQuantumWire*> outputs = {});

    virtual ~NInMOutQuantumGate() = default;

    size_t getNumInputs() const override;

    size_t getNumOutputs() const override;

    AbstractQuantumWire* getInput(size_t idx) const override;

    AbstractQuantumWire* getOutput(size_t idx) const override;

    void setAtInput(size_t idx, AbstractQuantumWire* value) override;

    void setAtOutput(size_t idx, AbstractQuantumWire* value) override;

    bool isFullyConnected() const override
    {
        for (const auto input : mInputs)
        {
            if (input == nullptr)
            {
                return false;
            }
        }

        for (const auto output : mOutputs)
        {
            if (output == nullptr)
            {
                return false;
            }
        }
        return true;
    }

private:
    size_t mNumIn{ 1 };
    size_t mNumOut{ 1 };

    Vector<AbstractQuantumWire*> mInputs;
    Vector<AbstractQuantumWire*> mOutputs;
};

class TwoInOneOutQuantumGate : public NInMOutQuantumGate
{
public:
    TwoInOneOutQuantumGate(AbstractQuantumWire* input0 = nullptr, AbstractQuantumWire* input1 = nullptr, AbstractQuantumWire* output = nullptr);

    void setInput0(AbstractQuantumWire* input);

    void setInput1(AbstractQuantumWire* input);

    void setOutput(AbstractQuantumWire* output);
};

class OneInOneOutQuantumGate : public NInMOutQuantumGate
{
public:
    OneInOneOutQuantumGate(AbstractQuantumWire* input = nullptr, AbstractQuantumWire* output = nullptr);

    void setInput(AbstractQuantumWire* input);

    void setOutput(AbstractQuantumWire* output);
};